; Mk14 VDU PIC emulation
;
; K Orton 2020

; Emulates the Science of Cambridge Mk14 VDU
; Part = PIC16F877
; FOsc = 16MHz
;
; Port A assignment:
;	RA0	/ Address bus A8..A11
;	RA1    /
;	RA2   /
;	RA3  /
;	RA4	NRDS
;	RA5	NENIN

; Port B assignment
;	RB0         / Data bus D0..D7
;	RB1        /
;	RB2       /
;	RB3      /
;	RB4     /
;	RB5    /
;	RB6   /
;	RB7  /

; Port C assignment
;	RC0	/VDUON *
;	RC1	GRPH/CHRS *
;	RC2	/SWAP *
;	RC3	/INVERT *
;	RC4	TOPPGE
;	RC5	/SYNC
;	RC6	PS4 **
;	RC7	PIXELS
;
; * 10k pull-up required
; ** 10k pull-up plus series 10k to port pin

; Port D assignment
;	RD0         / Address bus A0..A7
;	RD1        /
;	RD2       /
;	RD3      /
;	RD4     /
;	RD5    /
;	RD6   /
;	RD7  /

; Port E assignment
;	RE0	PS1 *
;	RE1	PS2 *
;	RE2	PS3 *
;
; * 10k pull-up required

; 625 line field structure (non-interlaced)
;
;	3 lines		Five broad pulses
;	19 lines	Blanking
;	16 lines	Top border
;	256 lines	Display area
;	16 lines	Bottom border
;	2 lines		Blanking
;	---------
;	312 lines	Total field duration

; 625 line line structure (monochrome)
;
;	19~		Line sync pulse
;	31~		Back porch
;	32~		Left border
;	128~		Display area
;	32~		Right border
;	14~		Front porch
;	---
;	256~		Total line duration

; PIC constants
W	EQU	0
INDF	EQU	H'0000'
PCL	EQU	H'0002'
STATUS	EQU	H'0003'
FSR	EQU	H'0004'
PORTA	EQU	H'0005'
PORTB	EQU	H'0006'
PORTC	EQU	H'0007'
PORTD	EQU	H'0008'
PORTE	EQU	H'0009'
PCLATH	EQU	H'000A'
RCSTA	EQU	H'0018'
TXREG	EQU	H'0019'
TRISA	EQU	H'0085'
TRISB	EQU	H'0086'
TRISC	EQU	H'0087'
TRISD	EQU	H'0088'
TRISE	EQU	H'0089'
TXSTA	EQU	H'0098'
SPBRG	EQU	H'0099'
ADCON1	EQU	H'009F'
C	EQU	0
Z	EQU	2
RP0	EQU	5
SPEN	EQU	7
CFG	EQU	H'3D32'

	__CONFIG	CFG

; Program constants
BUFA	EQU	H'0020'
BUFB	EQU	H'0030'
BUFC	EQU	H'0040'
BUFD	EQU	H'0050'
TPAGE	EQU	H'0060'
TFLAGS	EQU	H'0061'
TBGND	EQU	H'0062'
TINV	EQU	H'0063'
BPAGE	EQU	H'0064'
BFLAGS	EQU	H'0065'
BBGND	EQU	H'0066'
BINV	EQU	H'0067'
FNTPGE	EQU	H'0068'
PCHSAV	EQU	H'0069'
COUNTA	EQU	H'006A'
COUNTB	EQU	H'006B'
ADRSSH	EQU	PORTA
NRDS	EQU	4
NENIN	EQU	5
DBUS	EQU	PORTB
FLAGS	EQU	PORTC
VDUON	EQU	0
GRFCHR	EQU	1
SWAP	EQU	2
INVERT	EQU	3
TOPPGE	EQU	4
NSYNC	EQU	5
PS4	EQU	6
ADRSSL	EQU	PORTD
PS123	EQU	PORTE

; Delay macro (for up to 1023 cycles)
; Uses W for delays greater than 7~
DELAY	MACRO	CYCLES
	IF	((CYCLES>3)&&(CYCLES<8))
	CALL	DEL4
	ENDIF
	IF	(CYCLES>7)
	MOVLW	((CYCLES/4)-1)
	CALL	DEL4N
	ENDIF
	IF	((CYCLES-(4*(CYCLES/4)))>0)
	NOP
	ENDIF
	IF	((CYCLES-(4*(CYCLES/4)))>1)
	NOP
	ENDIF
	IF	((CYCLES-(4*(CYCLES/4)))>2)
	NOP
	ENDIF
	ENDM

; Read data from memory macro
RDMEM	MACRO
	BCF	PORTA,NRDS	; RDMEM = 7~
	DELAY	3
	MOVF	DBUS,W
	BSF	PORTA,NRDS
	INCF	ADRSSL
	ENDM

; Display character macro
; Assumes PCLATH points to font page
DSPCHR	MACRO	SCE		; DSPCHR = 8~
	MOVF	SCE,W
	CALL	H'80'
	MOVWF	TXREG
	ENDM

; Display pixels macro
; PCLATH must point to HIGH(BITREV)
; Index 0..7 expected in FNTPGE
DSPPIX	MACRO	SCE0, SCE1	; DSPPIX = 16~
	DELAY	6
	MOVF	SCE0,W
	BTFSC	FNTPGE,2
	MOVF	SCE1,W
	CALL	GTLU
	MOVWF	TXREG
	ENDM

; Reset
	ORG	0
	MOVLW	B'00010000'
	MOVWF	PORTA
	MOVLW	B'00100000'
	MOVWF	PORTC
	BSF	STATUS,RP0
	MOVLW	B'00011111'
	MOVWF	TRISA
	MOVLW	B'01001111'
	MOVWF	TRISC
	MOVLW	B'10110000'
	MOVWF	TXSTA
	CLRF	SPBRG
	MOVLW	B'00000110'
	MOVWF	ADCON1
	BCF	STATUS,RP0
	MOVLW	SPBRG
	MOVWF	FSR

; Main video loop
L1	MOVLW	5
	MOVWF	COUNTA
L2	BCF	PORTC,NSYNC
	DELAY	D'108'
	BSF	PORTC,NSYNC
	DELAY	D'15'
	DECFSZ	COUNTA
	GOTO	L2
	BSF	PORTC,TOPPGE
	BTFSS	PORTC,SWAP
	BCF	PORTC,TOPPGE
	DELAY	D'40'
	MOVF	PS123,W
	ANDLW	7
	BTFSC	PORTC,PS4
	IORLW	8
	MOVWF	TPAGE
	MOVF	FLAGS,W
	MOVWF	TFLAGS
	MOVLW	B'01000000'
	BTFSC	FLAGS,INVERT
	MOVLW	0
	MOVWF	TINV
	BCF	PORTC,TOPPGE
	BTFSS	PORTC,SWAP
	BSF	PORTC,TOPPGE
	DELAY	D'40'
	MOVF	PS123,W
	ANDLW	7
	BTFSC	PORTC,PS4
	IORLW	8
	MOVWF	BPAGE
	MOVF	FLAGS,W
	MOVWF	BFLAGS
	MOVLW	B'01000000'
	BTFSC	FLAGS,INVERT
	MOVLW	0
	MOVWF	BINV
	MOVLW	0
	BTFSS	TFLAGS,INVERT
	MOVLW	H'FF'
	MOVWF	TBGND
	MOVLW	0
	BTFSS	BFLAGS,INVERT
	MOVLW	H'FF'
	MOVWF	BBGND
	DELAY	D'10'
	BTFSS	FLAGS,VDUON
	GOTO	L5
	CLRF	COUNTA
L3	BCF	PORTC,NSYNC		; VDU off
	DELAY	D'18'
	BSF	PORTC,NSYNC
	DELAY	D'233'
	DECFSZ	COUNTA
	GOTO	L3
	DELAY	1
	BCF	PORTC,NSYNC
	DELAY	D'18'
	BSF	PORTC,NSYNC
	DELAY	D'234'
	MOVLW	D'51'
	MOVWF	COUNTA
L4	BCF	PORTC,NSYNC
	DELAY	D'18'
	BSF	PORTC,NSYNC
	DELAY	D'233'
	DECFSZ	COUNTA
	GOTO	L4
	DELAY	1
	BCF	PORTC,NSYNC
	DELAY	D'18'
	BSF	PORTC,NSYNC
	DELAY	D'232'
	GOTO	L1
L5	BCF	PORTC,NSYNC		; VDU on
	BSF	PORTA,NENIN
	DELAY	D'13'
	BSF	STATUS,RP0
	CLRF	TRISA
	CLRF	TRISD
	BCF	STATUS,RP0
	BSF	PORTC,NSYNC
	MOVF	TPAGE,W
	IORLW	B'00110000'
	MOVWF	ADRSSH
	CLRF	ADRSSL
	BTFSC	TFLAGS,GRFCHR		; Pre-charge BUFA
	GOTO	L55
	RDMEM
	ANDLW	B'00111111'
	IORWF	TINV,W
	MOVWF	BUFA+0
	RDMEM
	ANDLW	B'00111111'
	IORWF	TINV,W
	MOVWF	BUFA+1
	RDMEM
	ANDLW	B'00111111'
	IORWF	TINV,W
	MOVWF	BUFA+2
	RDMEM
	ANDLW	B'00111111'
	IORWF	TINV,W
	MOVWF	BUFA+3
	RDMEM
	ANDLW	B'00111111'
	IORWF	TINV,W
	MOVWF	BUFA+4
	RDMEM
	ANDLW	B'00111111'
	IORWF	TINV,W
	MOVWF	BUFA+5
	RDMEM
	ANDLW	B'00111111'
	IORWF	TINV,W
	MOVWF	BUFA+6
	RDMEM
	ANDLW	B'00111111'
	IORWF	TINV,W
	MOVWF	BUFA+7
	RDMEM
	ANDLW	B'00111111'
	IORWF	TINV,W
	MOVWF	BUFA+8
	RDMEM
	ANDLW	B'00111111'
	IORWF	TINV,W
	MOVWF	BUFA+9
	RDMEM
	ANDLW	B'00111111'
	IORWF	TINV,W
	MOVWF	BUFA+D'10'
	RDMEM
	ANDLW	B'00111111'
	IORWF	TINV,W
	MOVWF	BUFA+D'11'
	RDMEM
	ANDLW	B'00111111'
	IORWF	TINV,W
	MOVWF	BUFA+D'12'
	RDMEM
	ANDLW	B'00111111'
	IORWF	TINV,W
	MOVWF	BUFA+D'13'
	RDMEM
	ANDLW	B'00111111'
	IORWF	TINV,W
	MOVWF	BUFA+D'14'
	RDMEM
	ANDLW	B'00111111'
	IORWF	TINV,W
	MOVWF	BUFA+D'15'
	GOTO	L56
L55	RDMEM
	BTFSS	TFLAGS,INVERT
	XORLW	H'FF'
	MOVWF	BUFA+0
	RDMEM
	BTFSS	TFLAGS,INVERT
	XORLW	H'FF'
	MOVWF	BUFA+1
	RDMEM
	BTFSS	TFLAGS,INVERT
	XORLW	H'FF'
	MOVWF	BUFA+2
	RDMEM
	BTFSS	TFLAGS,INVERT
	XORLW	H'FF'
	MOVWF	BUFA+3
	RDMEM
	BTFSS	TFLAGS,INVERT
	XORLW	H'FF'
	MOVWF	BUFA+4
	RDMEM
	BTFSS	TFLAGS,INVERT
	XORLW	H'FF'
	MOVWF	BUFA+5
	RDMEM
	BTFSS	TFLAGS,INVERT
	XORLW	H'FF'
	MOVWF	BUFA+6
	RDMEM
	BTFSS	TFLAGS,INVERT
	XORLW	H'FF'
	MOVWF	BUFA+7
	RDMEM
	BTFSS	TFLAGS,INVERT
	XORLW	H'FF'
	MOVWF	BUFA+8
	RDMEM
	BTFSS	TFLAGS,INVERT
	XORLW	H'FF'
	MOVWF	BUFA+9
	RDMEM
	BTFSS	TFLAGS,INVERT
	XORLW	H'FF'
	MOVWF	BUFA+D'10'
	RDMEM
	BTFSS	TFLAGS,INVERT
	XORLW	H'FF'
	MOVWF	BUFA+D'11'
	RDMEM
	BTFSS	TFLAGS,INVERT
	XORLW	H'FF'
	MOVWF	BUFA+D'12'
	RDMEM
	BTFSS	TFLAGS,INVERT
	XORLW	H'FF'
	MOVWF	BUFA+D'13'
	RDMEM
	BTFSS	TFLAGS,INVERT
	XORLW	H'FF'
	MOVWF	BUFA+D'14'
	RDMEM
	BTFSS	TFLAGS,INVERT
	XORLW	H'FF'
	MOVWF	BUFA+D'15'
	DELAY	1
L56	DELAY	D'68'
	BCF	PORTC,NSYNC
	DELAY	D'18'
	BSF	PORTC,NSYNC
	MOVF	BPAGE,W
	IORLW	B'00110000'
	MOVWF	ADRSSH
	CLRF	ADRSSL
	BTFSC	BFLAGS,GRFCHR		; Pre-charge BUFC
	GOTO	L57
	RDMEM
	ANDLW	B'00111111'
	IORWF	BINV,W
	MOVWF	BUFC+0
	RDMEM
	ANDLW	B'00111111'
	IORWF	BINV,W
	MOVWF	BUFC+1
	RDMEM
	ANDLW	B'00111111'
	IORWF	BINV,W
	MOVWF	BUFC+2
	RDMEM
	ANDLW	B'00111111'
	IORWF	BINV,W
	MOVWF	BUFC+3
	RDMEM
	ANDLW	B'00111111'
	IORWF	BINV,W
	MOVWF	BUFC+4
	RDMEM
	ANDLW	B'00111111'
	IORWF	BINV,W
	MOVWF	BUFC+5
	RDMEM
	ANDLW	B'00111111'
	IORWF	BINV,W
	MOVWF	BUFC+6
	RDMEM
	ANDLW	B'00111111'
	IORWF	BINV,W
	MOVWF	BUFC+7
	RDMEM
	ANDLW	B'00111111'
	IORWF	BINV,W
	MOVWF	BUFC+8
	RDMEM
	ANDLW	B'00111111'
	IORWF	BINV,W
	MOVWF	BUFC+9
	RDMEM
	ANDLW	B'00111111'
	IORWF	BINV,W
	MOVWF	BUFC+D'10'
	RDMEM
	ANDLW	B'00111111'
	IORWF	BINV,W
	MOVWF	BUFC+D'11'
	RDMEM
	ANDLW	B'00111111'
	IORWF	BINV,W
	MOVWF	BUFC+D'12'
	RDMEM
	ANDLW	B'00111111'
	IORWF	BINV,W
	MOVWF	BUFC+D'13'
	RDMEM
	ANDLW	B'00111111'
	IORWF	BINV,W
	MOVWF	BUFC+D'14'
	RDMEM
	ANDLW	B'00111111'
	IORWF	BINV,W
	MOVWF	BUFC+D'15'
	GOTO	L58
L57	RDMEM
	BTFSS	BFLAGS,INVERT
	XORLW	H'FF'
	MOVWF	BUFC+0
	RDMEM
	BTFSS	BFLAGS,INVERT
	XORLW	H'FF'
	MOVWF	BUFC+1
	RDMEM
	BTFSS	BFLAGS,INVERT
	XORLW	H'FF'
	MOVWF	BUFC+2
	RDMEM
	BTFSS	BFLAGS,INVERT
	XORLW	H'FF'
	MOVWF	BUFC+3
	RDMEM
	BTFSS	BFLAGS,INVERT
	XORLW	H'FF'
	MOVWF	BUFC+4
	RDMEM
	BTFSS	BFLAGS,INVERT
	XORLW	H'FF'
	MOVWF	BUFC+5
	RDMEM
	BTFSS	BFLAGS,INVERT
	XORLW	H'FF'
	MOVWF	BUFC+6
	RDMEM
	BTFSS	BFLAGS,INVERT
	XORLW	H'FF'
	MOVWF	BUFC+7
	RDMEM
	BTFSS	BFLAGS,INVERT
	XORLW	H'FF'
	MOVWF	BUFC+8
	RDMEM
	BTFSS	BFLAGS,INVERT
	XORLW	H'FF'
	MOVWF	BUFC+9
	RDMEM
	BTFSS	BFLAGS,INVERT
	XORLW	H'FF'
	MOVWF	BUFC+D'10'
	RDMEM
	BTFSS	BFLAGS,INVERT
	XORLW	H'FF'
	MOVWF	BUFC+D'11'
	RDMEM
	BTFSS	BFLAGS,INVERT
	XORLW	H'FF'
	MOVWF	BUFC+D'12'
	RDMEM
	BTFSS	BFLAGS,INVERT
	XORLW	H'FF'
	MOVWF	BUFC+D'13'
	RDMEM
	BTFSS	BFLAGS,INVERT
	XORLW	H'FF'
	MOVWF	BUFC+D'14'
	RDMEM
	BTFSS	BFLAGS,INVERT
	XORLW	H'FF'
	MOVWF	BUFC+D'15'
	DELAY	1
L58	BSF	STATUS,RP0
	MOVLW	B'00011111'
	MOVWF	TRISA
	MOVLW	B'11111111'
	MOVWF	TRISD
	BCF	STATUS,RP0
	DELAY	D'19'
	BCF	PORTA,NENIN
	DELAY	D'40'
	MOVLW	D'16'
	MOVWF	COUNTA
L6	BCF	PORTC,NSYNC
	DELAY	D'18'
	BSF	PORTC,NSYNC
	DELAY	D'233'
	DECFSZ	COUNTA
	GOTO	L6
	DELAY	1
	BCF	PORTC,NSYNC
	DELAY	D'18'
	BSF	PORTC,NSYNC
	DELAY	D'234'
	MOVLW	D'15'
	MOVWF	COUNTA
L7	BCF	PORTC,NSYNC
	DELAY	D'18'
	BSF	PORTC,NSYNC
	DELAY	D'20'
	CLRF	INDF
	DELAY	7
	BSF	RCSTA,SPEN
	MOVF	TBGND,W
	MOVWF	TXREG
	MOVLW	D'23'
	MOVWF	COUNTB
L8	MOVF	TBGND,W
	MOVWF	TXREG
	DELAY	3
	DECFSZ	COUNTB
	GOTO	L8
	DELAY	6
	BCF	RCSTA,SPEN
	DELAY	D'10'
	DECFSZ	COUNTA
	GOTO	L7
	DELAY	1
	BCF	PORTC,NSYNC
	DELAY	D'18'
	BSF	PORTC,NSYNC
	BSF	PORTA,NENIN
	DELAY	D'19'
	BSF	STATUS,RP0
	CLRF	TRISA
	CLRF	TRISD
	BCF	STATUS,RP0
	DELAY	4
	BSF	RCSTA,SPEN
	MOVF	TBGND,W
	MOVWF	TXREG
	MOVLW	D'23'
	MOVWF	COUNTB
L9	MOVF	TBGND,W
	MOVWF	TXREG
	DELAY	3
	DECFSZ	COUNTB
	GOTO	L9
	BTFSC	TFLAGS,GRFCHR
	GOTO	L92
	BTFSC	BFLAGS,GRFCHR
	GOTO	L91
	DELAY	2
	BCF	RCSTA,SPEN
	DELAY	5
	MOVLW	HIGH(TOPTXT)
	MOVWF	PCLATH
	CALL	TOPTXT
	DELAY	1
	CALL	BTMTXT
	CLRF	PCLATH
	GOTO	L94
L91	DELAY	1
	BCF	RCSTA,SPEN
	DELAY	5
	MOVLW	HIGH(TOPTXT)
	MOVWF	PCLATH
	CALL	TOPTXT
	DELAY	1
	CALL	BTMGRF
	CLRF	PCLATH
	GOTO	L94
L92	BTFSC	BFLAGS,GRFCHR
	GOTO	L93
	DELAY	1
	BCF	RCSTA,SPEN
	DELAY	5
	MOVLW	HIGH(TOPGRF)
	MOVWF	PCLATH
	CALL	TOPGRF
	DELAY	1
	CALL	BTMTXT
	CLRF	PCLATH
	GOTO	L94
L93	BCF	RCSTA,SPEN
	DELAY	5
	MOVLW	HIGH(TOPGRF)
	MOVWF	PCLATH
	CALL	TOPGRF
	DELAY	1
	CALL	BTMGRF
	CLRF	PCLATH
	GOTO	L94
L94	CLRF	INDF
	DELAY	3
	BCF	PORTC,NSYNC
	BSF	STATUS,RP0
	MOVLW	B'00011111'
	MOVWF	TRISA
	MOVLW	B'11111111'
	MOVWF	TRISD
	BCF	STATUS,RP0
	DELAY	D'11'
	BCF	PORTA,NENIN
	BSF	PORTC,NSYNC
	DELAY	D'28'
	BSF	RCSTA,SPEN
	MOVF	BBGND,W
	MOVWF	TXREG
	MOVLW	D'23'
	MOVWF	COUNTB
L11	MOVF	BBGND,W
	MOVWF	TXREG
	DELAY	3
	DECFSZ	COUNTB
	GOTO	L11
	DELAY	6
	BCF	RCSTA,SPEN
	DELAY	D'11'
	MOVLW	D'15'
	MOVWF	COUNTA
L12	BCF	PORTC,NSYNC
	DELAY	D'18'
	BSF	PORTC,NSYNC
	DELAY	D'28'
	BSF	RCSTA,SPEN
	MOVF	BBGND,W
	MOVWF	TXREG
	MOVLW	D'23'
	MOVWF	COUNTB
L13	MOVF	BBGND,W
	MOVWF	TXREG
	DELAY	3
	DECFSZ	COUNTB
	GOTO	L13
	DELAY	6
	BCF	RCSTA,SPEN
	DELAY	D'10'
	DECFSZ	COUNTA
	GOTO	L12
	DELAY	1
	BCF	PORTC,NSYNC
	DELAY	D'18'
	BSF	PORTC,NSYNC
	DELAY	D'236'
	BCF	PORTC,NSYNC
	DELAY	D'18'
	BSF	PORTC,NSYNC
	DELAY	D'232'
	GOTO	L1

; Delay routines
	ORG	H'07FC'
DEL4N	ADDLW	H'FF'			; CALL DEL4N = ((4*(W+1))-1)~
	BTFSS	STATUS,Z
	GOTO	DEL4N
DEL4	RETURN				; CALL DEL4 = 4~

	ORG	H'0800'

; Top half text
TOPTXT	MOVLW	8			; CALL TOPTXT = 32767~
	MOVWF	COUNTA
	GOTO	L101
L100	DELAY	8
L101	BCF	PORTC,NSYNC
	MOVF	TPAGE,W
	IORLW	B'00110000'
	MOVWF	ADRSSH
	RDMEM
	ANDLW	B'00111111'
	IORWF	TINV,W
	MOVWF	BUFB+0
	DELAY	5
	BSF	PORTC,NSYNC
	MOVLW	HIGH(CFONT)
	MOVWF	FNTPGE
	CALL	RDTOPT
	MOVWF	BUFB+1
	CALL	DTXTA
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDTOPT
	MOVWF	BUFB+2
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDTOPT
	MOVWF	BUFB+3
	DELAY	1
	CALL	DTXTA
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDTOPT
	MOVWF	BUFB+4
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDTOPT
	MOVWF	BUFB+5
	DELAY	1
	CALL	DTXTA
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDTOPT
	MOVWF	BUFB+6
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDTOPT
	MOVWF	BUFB+7
	DELAY	1
	CALL	DTXTA
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDTOPT
	MOVWF	BUFB+8
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDTOPT
	MOVWF	BUFB+9
	DELAY	1
	CALL	DTXTA
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDTOPT
	MOVWF	BUFB+D'10'
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDTOPT
	MOVWF	BUFB+D'11'
	DELAY	1
	CALL	DTXTA
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDTOPT
	MOVWF	BUFB+D'12'
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDTOPT
	MOVWF	BUFB+D'13'
	DELAY	1
	CALL	DTXTA
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDTOPT
	MOVWF	BUFB+D'14'
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDTOPT
	MOVWF	BUFB+D'15'
	DELAY	1
	CALL	DTXTA
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDTOPT
	MOVWF	BUFA+0
	DELAY	4
	BSF	PORTC,NSYNC
	MOVLW	HIGH(CFONT)
	MOVWF	FNTPGE
	CALL	RDTOPT
	MOVWF	BUFA+1
	CALL	DTXTB
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDTOPT
	MOVWF	BUFA+2
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDTOPT
	MOVWF	BUFA+3
	DELAY	1
	CALL	DTXTB
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDTOPT
	MOVWF	BUFA+4
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDTOPT
	MOVWF	BUFA+5
	DELAY	1
	CALL	DTXTB
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDTOPT
	MOVWF	BUFA+6
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDTOPT
	MOVWF	BUFA+7
	DELAY	1
	CALL	DTXTB
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDTOPT
	MOVWF	BUFA+8
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDTOPT
	MOVWF	BUFA+9
	DELAY	1
	CALL	DTXTB
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDTOPT
	MOVWF	BUFA+D'10'
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDTOPT
	MOVWF	BUFA+D'11'
	DELAY	1
	CALL	DTXTB
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDTOPT
	MOVWF	BUFA+D'12'
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDTOPT
	MOVWF	BUFA+D'13'
	DELAY	1
	CALL	DTXTB
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDTOPT
	MOVWF	BUFA+D'14'
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDTOPT
	MOVWF	BUFA+D'15'
	DELAY	1
	CALL	DTXTB
	DECFSZ	COUNTA
	GOTO	L100
	RETURN

; Bottom half text
BTMTXT	MOVLW	8			; CALL BTMTXT = 32767~
	MOVWF	COUNTA
	GOTO	L201
L200	DELAY	8
L201	BCF	PORTC,NSYNC
	MOVF	BPAGE,W
	IORLW	B'00110000'
	MOVWF	ADRSSH
	RDMEM
	ANDLW	B'00111111'
	IORWF	BINV,W
	MOVWF	BUFD+0
	DELAY	5
	BSF	PORTC,NSYNC
	MOVLW	HIGH(CFONT)
	MOVWF	FNTPGE
	CALL	RDBTMT
	MOVWF	BUFD+1
	CALL	DTXTC
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDBTMT
	MOVWF	BUFD+2
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDBTMT
	MOVWF	BUFD+3
	DELAY	1
	CALL	DTXTC
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDBTMT
	MOVWF	BUFD+4
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDBTMT
	MOVWF	BUFD+5
	DELAY	1
	CALL	DTXTC
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDBTMT
	MOVWF	BUFD+6
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDBTMT
	MOVWF	BUFD+7
	DELAY	1
	CALL	DTXTC
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDBTMT
	MOVWF	BUFD+8
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDBTMT
	MOVWF	BUFD+9
	DELAY	1
	CALL	DTXTC
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDBTMT
	MOVWF	BUFD+D'10'
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDBTMT
	MOVWF	BUFD+D'11'
	DELAY	1
	CALL	DTXTC
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDBTMT
	MOVWF	BUFD+D'12'
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDBTMT
	MOVWF	BUFD+D'13'
	DELAY	1
	CALL	DTXTC
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDBTMT
	MOVWF	BUFD+D'14'
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDBTMT
	MOVWF	BUFD+D'15'
	DELAY	1
	CALL	DTXTC
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDBTMT
	MOVWF	BUFC+0
	DELAY	4
	BSF	PORTC,NSYNC
	MOVLW	HIGH(CFONT)
	MOVWF	FNTPGE
	CALL	RDBTMT
	MOVWF	BUFC+1
	CALL	DTXTD
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDBTMT
	MOVWF	BUFC+2
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDBTMT
	MOVWF	BUFC+3
	DELAY	1
	CALL	DTXTD
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDBTMT
	MOVWF	BUFC+4
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDBTMT
	MOVWF	BUFC+5
	DELAY	1
	CALL	DTXTD
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDBTMT
	MOVWF	BUFC+6
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDBTMT
	MOVWF	BUFC+7
	DELAY	1
	CALL	DTXTD
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDBTMT
	MOVWF	BUFC+8
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDBTMT
	MOVWF	BUFC+9
	DELAY	1
	CALL	DTXTD
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDBTMT
	MOVWF	BUFC+D'10'
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDBTMT
	MOVWF	BUFC+D'11'
	DELAY	1
	CALL	DTXTD
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDBTMT
	MOVWF	BUFC+D'12'
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDBTMT
	MOVWF	BUFC+D'13'
	DELAY	1
	CALL	DTXTD
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDBTMT
	MOVWF	BUFC+D'14'
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDBTMT
	MOVWF	BUFC+D'15'
	DELAY	1
	CALL	DTXTD
	DECFSZ	COUNTA
	GOTO	L200
	RETURN

; Display text from BUFA
; Expects font page in FNTPGE
DTXTA	CLRF	INDF
	CALL	OFFBUS
	BSF	RCSTA,SPEN			; CALL DTXTA = 209~
	MOVF	TBGND,W
	MOVWF	TXREG
	DELAY	3
	MOVWF	TXREG
	DELAY	7
	MOVWF	TXREG
	DELAY	1
	BCF	PORTA,NENIN
	MOVF	PCLATH,W
	MOVWF	PCHSAV
	MOVF	FNTPGE,W
	MOVWF	PCLATH
	MOVF	TBGND,W
	MOVWF	TXREG
	DSPCHR	BUFA+0
	DSPCHR	BUFA+1
	DSPCHR	BUFA+2
	DSPCHR	BUFA+3
	DSPCHR	BUFA+4
	DSPCHR	BUFA+5
	DSPCHR	BUFA+6
	DSPCHR	BUFA+7
	DSPCHR	BUFA+8
	DSPCHR	BUFA+9
	DSPCHR	BUFA+D'10'
	DSPCHR	BUFA+D'11'
	DSPCHR	BUFA+D'12'
	DSPCHR	BUFA+D'13'
	DSPCHR	BUFA+D'14'
	DSPCHR	BUFA+D'15'
	MOVF	PCHSAV,W
	MOVWF	PCLATH
	MOVF	TPAGE,W		; Replaces BSF PORTA,NENIN
	IORLW	B'00110000'
	MOVWF	PORTA
	DELAY	1
	MOVF	TBGND,W
	MOVWF	TXREG
	DELAY	7
	MOVWF	TXREG
	DELAY	7
	MOVWF	TXREG
	DELAY	7
	MOVWF	TXREG
	CALL	ONBUS
	BCF	RCSTA,SPEN
	RETURN

; Display text from BUFB
; Expects font page in FNTPGE
DTXTB	CLRF	INDF
	CALL	OFFBUS
	BSF	RCSTA,SPEN			; CALL DTXTB = 209~
	MOVF	TBGND,W
	MOVWF	TXREG
	DELAY	3
	MOVWF	TXREG
	DELAY	7
	MOVWF	TXREG
	DELAY	1
	BCF	PORTA,NENIN
	MOVF	PCLATH,W
	MOVWF	PCHSAV
	MOVF	FNTPGE,W
	MOVWF	PCLATH
	MOVF	TBGND,W
	MOVWF	TXREG
	DSPCHR	BUFB+0
	DSPCHR	BUFB+1
	DSPCHR	BUFB+2
	DSPCHR	BUFB+3
	DSPCHR	BUFB+4
	DSPCHR	BUFB+5
	DSPCHR	BUFB+6
	DSPCHR	BUFB+7
	DSPCHR	BUFB+8
	DSPCHR	BUFB+9
	DSPCHR	BUFB+D'10'
	DSPCHR	BUFB+D'11'
	DSPCHR	BUFB+D'12'
	DSPCHR	BUFB+D'13'
	DSPCHR	BUFB+D'14'
	DSPCHR	BUFB+D'15'
	MOVF	PCHSAV,W
	MOVWF	PCLATH
	MOVF	TPAGE,W		; Replaces BSF PORTA,NENIN
	IORLW	B'00110000'
	MOVWF	PORTA
	DELAY	1
	MOVF	TBGND,W
	MOVWF	TXREG
	DELAY	7
	MOVWF	TXREG
	DELAY	7
	MOVWF	TXREG
	DELAY	7
	MOVWF	TXREG
	CALL	ONBUS
	BCF	RCSTA,SPEN
	RETURN

; Display text from BUFC
; Expects font page in FNTPGE
DTXTC	CLRF	INDF
	CALL	OFFBUS
	BSF	RCSTA,SPEN			; CALL DTXTC = 209~
	MOVF	BBGND,W
	MOVWF	TXREG
	DELAY	3
	MOVWF	TXREG
	DELAY	7
	MOVWF	TXREG
	DELAY	1
	BCF	PORTA,NENIN
	MOVF	PCLATH,W
	MOVWF	PCHSAV
	MOVF	FNTPGE,W
	MOVWF	PCLATH
	MOVF	BBGND,W
	MOVWF	TXREG
	DSPCHR	BUFC+0
	DSPCHR	BUFC+1
	DSPCHR	BUFC+2
	DSPCHR	BUFC+3
	DSPCHR	BUFC+4
	DSPCHR	BUFC+5
	DSPCHR	BUFC+6
	DSPCHR	BUFC+7
	DSPCHR	BUFC+8
	DSPCHR	BUFC+9
	DSPCHR	BUFC+D'10'
	DSPCHR	BUFC+D'11'
	DSPCHR	BUFC+D'12'
	DSPCHR	BUFC+D'13'
	DSPCHR	BUFC+D'14'
	DSPCHR	BUFC+D'15'
	MOVF	PCHSAV,W
	MOVWF	PCLATH
	MOVF	BPAGE,W		; Replaces BSF PORTA,NENIN
	IORLW	B'00110000'
	MOVWF	PORTA
	DELAY	1
	MOVF	BBGND,W
	MOVWF	TXREG
	DELAY	7
	MOVWF	TXREG
	DELAY	7
	MOVWF	TXREG
	DELAY	7
	MOVWF	TXREG
	CALL	ONBUS
	BCF	RCSTA,SPEN
	RETURN

; Display text from BUFD
; Expects font page in FNTPGE
DTXTD	CLRF	INDF
	CALL	OFFBUS
	BSF	RCSTA,SPEN			; CALL DTXTD = 209~
	MOVF	BBGND,W
	MOVWF	TXREG
	DELAY	3
	MOVWF	TXREG
	DELAY	7
	MOVWF	TXREG
	DELAY	1
	BCF	PORTA,NENIN
	MOVF	PCLATH,W
	MOVWF	PCHSAV
	MOVF	FNTPGE,W
	MOVWF	PCLATH
	MOVF	BBGND,W
	MOVWF	TXREG
	DSPCHR	BUFD+0
	DSPCHR	BUFD+1
	DSPCHR	BUFD+2
	DSPCHR	BUFD+3
	DSPCHR	BUFD+4
	DSPCHR	BUFD+5
	DSPCHR	BUFD+6
	DSPCHR	BUFD+7
	DSPCHR	BUFD+8
	DSPCHR	BUFD+9
	DSPCHR	BUFD+D'10'
	DSPCHR	BUFD+D'11'
	DSPCHR	BUFD+D'12'
	DSPCHR	BUFD+D'13'
	DSPCHR	BUFD+D'14'
	DSPCHR	BUFD+D'15'
	MOVF	PCHSAV,W
	MOVWF	PCLATH
	MOVF	BPAGE,W		; Replaces BSF PORTA,NENIN
	IORLW	B'00110000'
	MOVWF	PORTA
	DELAY	1
	MOVF	BBGND,W
	MOVWF	TXREG
	DELAY	7
	MOVWF	TXREG
	DELAY	7
	MOVWF	TXREG
	DELAY	7
	MOVWF	TXREG
	CALL	ONBUS
	BCF	RCSTA,SPEN
	RETURN

; Get off bus
OFFBUS	BSF	STATUS,RP0		; CALL OFFBUS = 9~
	MOVLW	B'00011111'
	MOVWF	TRISA
	COMF	TRISD
	BCF	STATUS,RP0
	RETURN

; Get back on bus
ONBUS	DELAY	3			; CALL ONBUS = 11~
	BSF	STATUS,RP0
	CLRF	TRISA
	CLRF	TRISD
	BCF	STATUS,RP0
	RETURN

; Read text data for top
RDTOPT	RDMEM				; CALL RDTOPT = 13~
	ANDLW	B'00111111'
	IORWF	TINV,W
	RETURN

; Read text data for bottom
RDBTMT	RDMEM				; CALL RDBTMT = 13~
	ANDLW	B'00111111'
	IORWF	BINV,W
	RETURN

; Top half graphics
TOPGRF	MOVLW	8			; CALL TOPGRF = 32767~
	MOVWF	COUNTA
	GOTO	L301
L300	DELAY	8
L301	BCF	PORTC,NSYNC
	MOVF	TPAGE,W
	IORLW	B'00110000'
	MOVWF	ADRSSH
	RDMEM
	BTFSS	TFLAGS,INVERT
	XORLW	H'FF'
	MOVWF	BUFB+0
	DELAY	5
	BSF	PORTC,NSYNC
	CLRF	FNTPGE
	CALL	RDTOPG
	MOVWF	BUFB+1
	CALL	DGRFA
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDTOPG
	MOVWF	BUFB+2
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDTOPG
	MOVWF	BUFB+3
	CALL	DGRFA
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDTOPG
	MOVWF	BUFB+4
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDTOPG
	MOVWF	BUFB+5
	CALL	DGRFA
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDTOPG
	MOVWF	BUFB+6
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDTOPG
	MOVWF	BUFB+7
	CALL	DGRFA
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDTOPG
	MOVWF	BUFB+8
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDTOPG
	MOVWF	BUFB+9
	CALL	DGRFA
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDTOPG
	MOVWF	BUFB+D'10'
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDTOPG
	MOVWF	BUFB+D'11'
	CALL	DGRFA
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDTOPG
	MOVWF	BUFB+D'12'
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDTOPG
	MOVWF	BUFB+D'13'
	CALL	DGRFA
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDTOPG
	MOVWF	BUFB+D'14'
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDTOPG
	MOVWF	BUFB+D'15'
	CALL	DGRFA
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDTOPG
	MOVWF	BUFA+0
	DELAY	4
	BSF	PORTC,NSYNC
	CLRF	FNTPGE
	CALL	RDTOPG
	MOVWF	BUFA+1
	CALL	DGRFB
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDTOPG
	MOVWF	BUFA+2
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDTOPG
	MOVWF	BUFA+3
	CALL	DGRFB
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDTOPG
	MOVWF	BUFA+4
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDTOPG
	MOVWF	BUFA+5
	CALL	DGRFB
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDTOPG
	MOVWF	BUFA+6
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDTOPG
	MOVWF	BUFA+7
	CALL	DGRFB
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDTOPG
	MOVWF	BUFA+8
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDTOPG
	MOVWF	BUFA+9
	CALL	DGRFB
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDTOPG
	MOVWF	BUFA+D'10'
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDTOPG
	MOVWF	BUFA+D'11'
	CALL	DGRFB
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDTOPG
	MOVWF	BUFA+D'12'
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDTOPG
	MOVWF	BUFA+D'13'
	CALL	DGRFB
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDTOPG
	MOVWF	BUFA+D'14'
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDTOPG
	MOVWF	BUFA+D'15'
	CALL	DGRFB
	DECFSZ	COUNTA
	GOTO	L300
	RETURN

; Bottom half graphics
BTMGRF	MOVLW	8			; CALL BTMGRF = 32767~
	MOVWF	COUNTA
	GOTO	L401
L400	DELAY	8
L401	BCF	PORTC,NSYNC
	MOVF	BPAGE,W
	IORLW	B'00110000'
	MOVWF	ADRSSH
	RDMEM
	BTFSS	BFLAGS,INVERT
	XORLW	H'FF'
	MOVWF	BUFD+0
	DELAY	5
	BSF	PORTC,NSYNC
	CLRF	FNTPGE
	CALL	RDBTMG
	MOVWF	BUFD+1
	CALL	DGRFC
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDBTMG
	MOVWF	BUFD+2
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDBTMG
	MOVWF	BUFD+3
	CALL	DGRFC
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDBTMG
	MOVWF	BUFD+4
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDBTMG
	MOVWF	BUFD+5
	CALL	DGRFC
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDBTMG
	MOVWF	BUFD+6
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDBTMG
	MOVWF	BUFD+7
	CALL	DGRFC
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDBTMG
	MOVWF	BUFD+8
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDBTMG
	MOVWF	BUFD+9
	CALL	DGRFC
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDBTMG
	MOVWF	BUFD+D'10'
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDBTMG
	MOVWF	BUFD+D'11'
	CALL	DGRFC
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDBTMG
	MOVWF	BUFD+D'12'
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDBTMG
	MOVWF	BUFD+D'13'
	CALL	DGRFC
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDBTMG
	MOVWF	BUFD+D'14'
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDBTMG
	MOVWF	BUFD+D'15'
	CALL	DGRFC
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDBTMG
	MOVWF	BUFC+0
	DELAY	4
	BSF	PORTC,NSYNC
	CLRF	FNTPGE
	CALL	RDBTMG
	MOVWF	BUFC+1
	CALL	DGRFD
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDBTMG
	MOVWF	BUFC+2
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDBTMG
	MOVWF	BUFC+3
	CALL	DGRFD
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDBTMG
	MOVWF	BUFC+4
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDBTMG
	MOVWF	BUFC+5
	CALL	DGRFD
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDBTMG
	MOVWF	BUFC+6
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDBTMG
	MOVWF	BUFC+7
	CALL	DGRFD
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDBTMG
	MOVWF	BUFC+8
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDBTMG
	MOVWF	BUFC+9
	CALL	DGRFD
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDBTMG
	MOVWF	BUFC+D'10'
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDBTMG
	MOVWF	BUFC+D'11'
	CALL	DGRFD
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDBTMG
	MOVWF	BUFC+D'12'
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDBTMG
	MOVWF	BUFC+D'13'
	CALL	DGRFD
	DELAY	D'11'
	BCF	PORTC,NSYNC
	CALL	RDBTMG
	MOVWF	BUFC+D'14'
	DELAY	4
	BSF	PORTC,NSYNC
	INCF	FNTPGE
	CALL	RDBTMG
	MOVWF	BUFC+D'15'
	CALL	DGRFD
	DECFSZ	COUNTA
	GOTO	L400
	RETURN

; Display graphics from BUFA
; Expects index 0..7 in FNTPGE
DGRFA	BSF	INDF,0				; CALL DGRFA = 210~
	CALL	OFFBUS
	BSF	RCSTA,SPEN
	DELAY	1
	MOVF	TBGND,W
	MOVWF	TXREG
	MOVF	PCLATH,W
	MOVWF	PCHSAV
	MOVLW	HIGH(BITREV)
	MOVWF	PCLATH
	DELAY	1
	BCF	PORTA,NENIN
	MOVF	TBGND,W
	MOVWF	TXREG
	DSPPIX	BUFA+0,BUFA+8
	DSPPIX	BUFA+1,BUFA+9
	DSPPIX	BUFA+2,BUFA+D'10'
	DSPPIX	BUFA+3,BUFA+D'11'
	DSPPIX	BUFA+4,BUFA+D'12'
	DSPPIX	BUFA+5,BUFA+D'13'
	DSPPIX	BUFA+6,BUFA+D'14'
	DSPPIX	BUFA+7,BUFA+D'15'
	MOVF	PCHSAV,W
	MOVWF	PCLATH
	MOVF	TPAGE,W		; Replaces BSF PORTA,NENIN
	IORLW	B'00110000'
	MOVWF	PORTA
	DELAY	9
	MOVF	TBGND,W
	MOVWF	TXREG
	DELAY	D'14'
	MOVF	TBGND,W
	MOVWF	TXREG
	DELAY	D'12'
	CALL	ONBUS
	BCF	RCSTA,SPEN
	RETURN

; Display graphics from BUFB
; Expects index 0..7 in FNTPGE
DGRFB	BSF	INDF,0				; CALL DGRFB = 210~
	CALL	OFFBUS
	BSF	RCSTA,SPEN
	DELAY	1
	MOVF	TBGND,W
	MOVWF	TXREG
	MOVF	PCLATH,W
	MOVWF	PCHSAV
	MOVLW	HIGH(BITREV)
	MOVWF	PCLATH
	DELAY	1
	BCF	PORTA,NENIN
	MOVF	TBGND,W
	MOVWF	TXREG
	DSPPIX	BUFB+0,BUFB+8
	DSPPIX	BUFB+1,BUFB+9
	DSPPIX	BUFB+2,BUFB+D'10'
	DSPPIX	BUFB+3,BUFB+D'11'
	DSPPIX	BUFB+4,BUFB+D'12'
	DSPPIX	BUFB+5,BUFB+D'13'
	DSPPIX	BUFB+6,BUFB+D'14'
	DSPPIX	BUFB+7,BUFB+D'15'
	MOVF	PCHSAV,W
	MOVWF	PCLATH
	MOVF	TPAGE,W		; Replaces BSF PORTA,NENIN
	IORLW	B'00110000'
	MOVWF	PORTA
	DELAY	9
	MOVF	TBGND,W
	MOVWF	TXREG
	DELAY	D'14'
	MOVF	TBGND,W
	MOVWF	TXREG
	DELAY	D'12'
	CALL	ONBUS
	BCF	RCSTA,SPEN
	RETURN

; Display graphics from BUFC
; Expects index 0..7 in FNTPGE
DGRFC	BSF	INDF,0				; CALL DGRFC = 210~
	CALL	OFFBUS
	BSF	RCSTA,SPEN
	DELAY	1
	MOVF	BBGND,W
	MOVWF	TXREG
	MOVF	PCLATH,W
	MOVWF	PCHSAV
	MOVLW	HIGH(BITREV)
	MOVWF	PCLATH
	DELAY	1
	BCF	PORTA,NENIN
	MOVF	BBGND,W
	MOVWF	TXREG
	DSPPIX	BUFC+0,BUFC+8
	DSPPIX	BUFC+1,BUFC+9
	DSPPIX	BUFC+2,BUFC+D'10'
	DSPPIX	BUFC+3,BUFC+D'11'
	DSPPIX	BUFC+4,BUFC+D'12'
	DSPPIX	BUFC+5,BUFC+D'13'
	DSPPIX	BUFC+6,BUFC+D'14'
	DSPPIX	BUFC+7,BUFC+D'15'
	MOVF	PCHSAV,W
	MOVWF	PCLATH
	MOVF	BPAGE,W		; Replaces BSF PORTA,NENIN
	IORLW	B'00110000'
	MOVWF	PORTA
	DELAY	9
	MOVF	BBGND,W
	MOVWF	TXREG
	DELAY	D'14'
	MOVF	BBGND,W
	MOVWF	TXREG
	DELAY	D'12'
	CALL	ONBUS
	BCF	RCSTA,SPEN
	RETURN

; Display graphics from BUFD
; Expects index 0..7 in FNTPGE
DGRFD	BSF	INDF,0				; CALL DGRFD = 210~
	CALL	OFFBUS
	BSF	RCSTA,SPEN
	DELAY	1
	MOVF	BBGND,W
	MOVWF	TXREG
	MOVF	PCLATH,W
	MOVWF	PCHSAV
	MOVLW	HIGH(BITREV)
	MOVWF	PCLATH
	DELAY	1
	BCF	PORTA,NENIN
	MOVF	BBGND,W
	MOVWF	TXREG
	DSPPIX	BUFD+0,BUFD+8
	DSPPIX	BUFD+1,BUFD+9
	DSPPIX	BUFD+2,BUFD+D'10'
	DSPPIX	BUFD+3,BUFD+D'11'
	DSPPIX	BUFD+4,BUFD+D'12'
	DSPPIX	BUFD+5,BUFD+D'13'
	DSPPIX	BUFD+6,BUFD+D'14'
	DSPPIX	BUFD+7,BUFD+D'15'
	MOVF	PCHSAV,W
	MOVWF	PCLATH
	MOVF	BPAGE,W		; Replaces BSF PORTA,NENIN
	IORLW	B'00110000'
	MOVWF	PORTA
	DELAY	9
	MOVF	BBGND,W
	MOVWF	TXREG
	DELAY	D'14'
	MOVF	BBGND,W
	MOVWF	TXREG
	DELAY	D'12'
	CALL	ONBUS
	BCF	RCSTA,SPEN
	RETURN

; Read graphics data for top
RDTOPG	RDMEM				; CALL RDTOPG = 13~
	BTFSS	TFLAGS,INVERT
	XORLW	H'FF'
	RETURN

; Read graphics data for bottom
RDBTMG	RDMEM				; CALL RDBTMG = 13~
	BTFSS	BFLAGS,INVERT
	XORLW	H'FF'
	RETURN

; Delay routines
	ORG	H'0FFC'
	ADDLW	H'FF'
	BTFSS	STATUS,Z
	GOTO	DEL4N
	RETURN

; Character font tables
	ORG	H'1000'
CFONT	DT	B'00011100',B'00001000',B'00011110',B'00011100',B'00011110',B'00111110',B'00111110',B'00111100'
	DT	B'00100010',B'00011100',B'00100000',B'00100010',B'00000010',B'00100010',B'00100010',B'00011100'
	DT	B'00011110',B'00011100',B'00011110',B'00011100',B'00111110',B'00100010',B'00100010',B'00100010'
	DT	B'00100010',B'00100010',B'00111110',B'00111110',B'00000000',B'00111110',B'00000000',B'00000000'
	DT	B'00000000',B'00001000',B'00010100',B'00010100',B'00001000',B'00000110',B'00000100',B'00001000'
	DT	B'00001000',B'00001000',B'00001000',B'00000000',B'00000000',B'00000000',B'00000000',B'00000000'
	DT	B'00011100',B'00001000',B'00011100',B'00111110',B'00010000',B'00111110',B'00111000',B'00111110'
	DT	B'00011100',B'00011100',B'00000000',B'00000000',B'00010000',B'00000000',B'00000100',B'00011100'
	DT	B'11100011',B'11110111',B'11100001',B'11100011',B'11100001',B'11000001',B'11000001',B'11000011'
	DT	B'11011101',B'11100011',B'11011111',B'11011101',B'11111101',B'11011101',B'11011101',B'11100011'
	DT	B'11100001',B'11100011',B'11100001',B'11100011',B'11000001',B'11011101',B'11011101',B'11011101'
	DT	B'11011101',B'11011101',B'11000001',B'11000001',B'11111111',B'11000001',B'11111111',B'11111111'
	DT	B'11111111',B'11110111',B'11101011',B'11101011',B'11110111',B'11111001',B'11111011',B'11110111'
	DT	B'11110111',B'11110111',B'11110111',B'11111111',B'11111111',B'11111111',B'11111111',B'11111111'
	DT	B'11100011',B'11110111',B'11100011',B'11000001',B'11101111',B'11000001',B'11000111',B'11000001'
	DT	B'11100011',B'11100011',B'11111111',B'11111111',B'11101111',B'11111111',B'11111011',B'11100011'
	MOVWF	PCL
	ORG	H'1100'
	DT	B'00100010',B'00010100',B'00100010',B'00100010',B'00100010',B'00000010',B'00000010',B'00000010'
	DT	B'00100010',B'00001000',B'00100000',B'00010010',B'00000010',B'00110110',B'00100010',B'00100010'
	DT	B'00100010',B'00100010',B'00100010',B'00100010',B'00001000',B'00100010',B'00100010',B'00100010'
	DT	B'00100010',B'00100010',B'00100000',B'00000110',B'00000010',B'00110000',B'00000000',B'00000000'
	DT	B'00000000',B'00001000',B'00010100',B'00010100',B'00111100',B'00100110',B'00001010',B'00001000'
	DT	B'00000100',B'00010000',B'00101010',B'00001000',B'00000000',B'00000000',B'00000000',B'00100000'
	DT	B'00100010',B'00001100',B'00100010',B'00100000',B'00011000',B'00000010',B'00000100',B'00100000'
	DT	B'00100010',B'00100010',B'00000000',B'00000000',B'00001000',B'00000000',B'00001000',B'00100010'
	DT	B'11011101',B'11101011',B'11011101',B'11011101',B'11011101',B'11111101',B'11111101',B'11111101'
	DT	B'11011101',B'11110111',B'11011111',B'11101101',B'11111101',B'11001001',B'11011101',B'11011101'
	DT	B'11011101',B'11011101',B'11011101',B'11011101',B'11110111',B'11011101',B'11011101',B'11011101'
	DT	B'11011101',B'11011101',B'11011111',B'11111001',B'11111101',B'11001111',B'11111111',B'11111111'
	DT	B'11111111',B'11110111',B'11101011',B'11101011',B'11000011',B'11011001',B'11110101',B'11110111'
	DT	B'11111011',B'11101111',B'11010101',B'11110111',B'11111111',B'11111111',B'11111111',B'11011111'
	DT	B'11011101',B'11110011',B'11011101',B'11011111',B'11100111',B'11111101',B'11111011',B'11011111'
	DT	B'11011101',B'11011101',B'11111111',B'11111111',B'11110111',B'11111111',B'11110111',B'11011101'
	MOVWF	PCL
	ORG	H'1200'
	DT	B'00101010',B'00100010',B'00100010',B'00000010',B'00100010',B'00000010',B'00000010',B'00000010'
	DT	B'00100010',B'00001000',B'00100000',B'00001010',B'00000010',B'00101010',B'00100110',B'00100010'
	DT	B'00100010',B'00100010',B'00100010',B'00000010',B'00001000',B'00100010',B'00100010',B'00100010'
	DT	B'00010100',B'00010100',B'00010000',B'00000110',B'00000100',B'00110000',B'00000000',B'00000000'
	DT	B'00000000',B'00001000',B'00010100',B'00111110',B'00001010',B'00010000',B'00001010',B'00001000'
	DT	B'00000010',B'00100000',B'00011100',B'00001000',B'00000000',B'00000000',B'00000000',B'00010000'
	DT	B'00110010',B'00001000',B'00100000',B'00010000',B'00010100',B'00011110',B'00000010',B'00100000'
	DT	B'00100010',B'00100010',B'00001000',B'00001000',B'00000100',B'00111110',B'00010000',B'00100000'
	DT	B'11010101',B'11011101',B'11011101',B'11111101',B'11011101',B'11111101',B'11111101',B'11111101'
	DT	B'11011101',B'11110111',B'11011111',B'11110101',B'11111101',B'11010101',B'11011001',B'11011101'
	DT	B'11011101',B'11011101',B'11011101',B'11111101',B'11110111',B'11011101',B'11011101',B'11011101'
	DT	B'11101011',B'11101011',B'11101111',B'11111001',B'11111011',B'11001111',B'11111111',B'11111111'
	DT	B'11111111',B'11110111',B'11101011',B'11000001',B'11110101',B'11101111',B'11110101',B'11110111'
	DT	B'11111101',B'11011111',B'11100011',B'11110111',B'11111111',B'11111111',B'11111111',B'11101111'
	DT	B'11001101',B'11110111',B'11011111',B'11101111',B'11101011',B'11100001',B'11111101',B'11011111'
	DT	B'11011101',B'11011101',B'11110111',B'11110111',B'11111011',B'11000001',B'11101111',B'11011111'
	MOVWF	PCL
	ORG	H'1300'
	DT	B'00111010',B'00100010',B'00011110',B'00000010',B'00100010',B'00011110',B'00011110',B'00000010'
	DT	B'00111110',B'00001000',B'00100000',B'00000110',B'00000010',B'00101010',B'00101010',B'00100010'
	DT	B'00011110',B'00100010',B'00011110',B'00011100',B'00001000',B'00100010',B'00010100',B'00101010'
	DT	B'00001000',B'00001000',B'00001000',B'00000110',B'00001000',B'00110000',B'00011100',B'00000000'
	DT	B'00000000',B'00001000',B'00000000',B'00010100',B'00011100',B'00001000',B'00000100',B'00000000'
	DT	B'00000010',B'00100000',B'00001000',B'00111110',B'00000000',B'00111110',B'00000000',B'00001000'
	DT	B'00101010',B'00001000',B'00011100',B'00011000',B'00010010',B'00100000',B'00011110',B'00010000'
	DT	B'00011100',B'00111100',B'00000000',B'00000000',B'00000010',B'00000000',B'00100000',B'00011000'
	DT	B'11000101',B'11011101',B'11100001',B'11111101',B'11011101',B'11100001',B'11100001',B'11111101'
	DT	B'11000001',B'11110111',B'11011111',B'11111001',B'11111101',B'11010101',B'11010101',B'11011101'
	DT	B'11100001',B'11011101',B'11100001',B'11100011',B'11110111',B'11011101',B'11101011',B'11010101'
	DT	B'11110111',B'11110111',B'11110111',B'11111001',B'11110111',B'11001111',B'11100011',B'11111111'
	DT	B'11111111',B'11110111',B'11111111',B'11101011',B'11100011',B'11110111',B'11111011',B'11111111'
	DT	B'11111101',B'11011111',B'11110111',B'11000001',B'11111111',B'11000001',B'11111111',B'11110111'
	DT	B'11010101',B'11110111',B'11100011',B'11100111',B'11101101',B'11011111',B'11100001',B'11101111'
	DT	B'11100011',B'11000011',B'11111111',B'11111111',B'11111101',B'11111111',B'11011111',B'11100111'
	MOVWF	PCL
	ORG	H'1400'
	DT	B'00011010',B'00111110',B'00100010',B'00000010',B'00100010',B'00000010',B'00000010',B'00110010'
	DT	B'00100010',B'00001000',B'00100000',B'00001010',B'00000010',B'00101010',B'00110010',B'00100010'
	DT	B'00000010',B'00101010',B'00001010',B'00100000',B'00001000',B'00100010',B'00010100',B'00101010'
	DT	B'00010100',B'00001000',B'00000100',B'00000110',B'00010000',B'00110000',B'00100010',B'00000000'
	DT	B'00000000',B'00001000',B'00000000',B'00111110',B'00101000',B'00000100',B'00101010',B'00000000'
	DT	B'00000010',B'00100000',B'00011100',B'00001000',B'00001000',B'00000000',B'00000000',B'00000100'
	DT	B'00100110',B'00001000',B'00000010',B'00100000',B'00111110',B'00100000',B'00100010',B'00001000'
	DT	B'00100010',B'00100000',B'00001000',B'00001000',B'00000100',B'00111110',B'00010000',B'00001000'
	DT	B'11100101',B'11000001',B'11011101',B'11111101',B'11011101',B'11111101',B'11111101',B'11001101'
	DT	B'11011101',B'11110111',B'11011111',B'11110101',B'11111101',B'11010101',B'11001101',B'11011101'
	DT	B'11111101',B'11010101',B'11110101',B'11011111',B'11110111',B'11011101',B'11101011',B'11010101'
	DT	B'11101011',B'11110111',B'11111011',B'11111001',B'11101111',B'11001111',B'11011101',B'11111111'
	DT	B'11111111',B'11110111',B'11111111',B'11000001',B'11010111',B'11111011',B'11010101',B'11111111'
	DT	B'11111101',B'11011111',B'11100011',B'11110111',B'11110111',B'11111111',B'11111111',B'11111011'
	DT	B'11011001',B'11110111',B'11111101',B'11011111',B'11000001',B'11011111',B'11011101',B'11110111'
	DT	B'11011101',B'11011111',B'11110111',B'11110111',B'11111011',B'11000001',B'11101111',B'11110111'
	MOVWF	PCL
	ORG	H'1500'
	DT	B'00000010',B'00100010',B'00100010',B'00100010',B'00100010',B'00000010',B'00000010',B'00100010'
	DT	B'00100010',B'00001000',B'00100010',B'00010010',B'00000010',B'00100010',B'00100010',B'00100010'
	DT	B'00000010',B'00010010',B'00010010',B'00100010',B'00001000',B'00100010',B'00001000',B'00101010'
	DT	B'00100010',B'00001000',B'00000010',B'00000110',B'00100000',B'00110000',B'00000000',B'00000000'
	DT	B'00000000',B'00000000',B'00000000',B'00010100',B'00011110',B'00110010',B'00010010',B'00000000'
	DT	B'00000100',B'00010000',B'00101010',B'00001000',B'00001000',B'00000000',B'00000000',B'00000010'
	DT	B'00100010',B'00001000',B'00000010',B'00100010',B'00010000',B'00100010',B'00100010',B'00000100'
	DT	B'00100010',B'00010000',B'00000000',B'00001000',B'00001000',B'00000000',B'00001000',B'00000000'
	DT	B'11111101',B'11011101',B'11011101',B'11011101',B'11011101',B'11111101',B'11111101',B'11011101'
	DT	B'11011101',B'11110111',B'11011101',B'11101101',B'11111101',B'11011101',B'11011101',B'11011101'
	DT	B'11111101',B'11101101',B'11101101',B'11011101',B'11110111',B'11011101',B'11110111',B'11010101'
	DT	B'11011101',B'11110111',B'11111101',B'11111001',B'11011111',B'11001111',B'11111111',B'11111111'
	DT	B'11111111',B'11111111',B'11111111',B'11101011',B'11100001',B'11001101',B'11101101',B'11111111'
	DT	B'11111011',B'11101111',B'11010101',B'11110111',B'11110111',B'11111111',B'11111111',B'11111101'
	DT	B'11011101',B'11110111',B'11111101',B'11011101',B'11101111',B'11011101',B'11011101',B'11111011'
	DT	B'11011101',B'11101111',B'11111111',B'11110111',B'11110111',B'11111111',B'11110111',B'11111111'
	MOVWF	PCL
	ORG	H'1600'
	DT	B'00111100',B'00100010',B'00011110',B'00011100',B'00011110',B'00111110',B'00000010',B'00111100'
	DT	B'00100010',B'00011100',B'00011100',B'00100010',B'00111110',B'00100010',B'00100010',B'00011100'
	DT	B'00000010',B'00101100',B'00100010',B'00011100',B'00001000',B'00011100',B'00001000',B'00010100'
	DT	B'00100010',B'00001000',B'00111110',B'00111110',B'00000000',B'00111110',B'00000000',B'00111110'
	DT	B'00000000',B'00001000',B'00000000',B'00010100',B'00001000',B'00110000',B'00101100',B'00000000'
	DT	B'00001000',B'00001000',B'00001000',B'00000000',B'00000100',B'00000000',B'00001000',B'00000000'
	DT	B'00011100',B'00011100',B'00111110',B'00011100',B'00010000',B'00011100',B'00011100',B'00000010'
	DT	B'00011100',B'00001110',B'00000000',B'00000100',B'00010000',B'00000000',B'00000100',B'00001000'
	DT	B'11000011',B'11011101',B'11100001',B'11100011',B'11100001',B'11000001',B'11111101',B'11000011'
	DT	B'11011101',B'11100011',B'11100011',B'11011101',B'11000001',B'11011101',B'11011101',B'11100011'
	DT	B'11111101',B'11010011',B'11011101',B'11100011',B'11110111',B'11100011',B'11110111',B'11101011'
	DT	B'11011101',B'11110111',B'11000001',B'11000001',B'11111111',B'11000001',B'11111111',B'11000001'
	DT	B'11111111',B'11110111',B'11111111',B'11101011',B'11110111',B'11001111',B'11010011',B'11111111'
	DT	B'11110111',B'11110111',B'11110111',B'11111111',B'11111011',B'11111111',B'11110111',B'11111111'
	DT	B'11100011',B'11100011',B'11000001',B'11100011',B'11101111',B'11100011',B'11100011',B'11111101'
	DT	B'11100011',B'11110001',B'11111111',B'11111011',B'11101111',B'11111111',B'11111011',B'11110111'
	MOVWF	PCL
	ORG	H'1700'
	DT	B'00000000',B'00000000',B'00000000',B'00000000',B'00000000',B'00000000',B'00000000',B'00000000'
	DT	B'00000000',B'00000000',B'00000000',B'00000000',B'00000000',B'00000000',B'00000000',B'00000000'
	DT	B'00000000',B'00000000',B'00000000',B'00000000',B'00000000',B'00000000',B'00000000',B'00000000'
	DT	B'00000000',B'00000000',B'00000000',B'00000000',B'00000000',B'00000000',B'00000000',B'00000000'
	DT	B'00000000',B'00000000',B'00000000',B'00000000',B'00000000',B'00000000',B'00000000',B'00000000'
	DT	B'00000000',B'00000000',B'00000000',B'00000000',B'00000000',B'00000000',B'00000000',B'00000000'
	DT	B'00000000',B'00000000',B'00000000',B'00000000',B'00000000',B'00000000',B'00000000',B'00000000'
	DT	B'00000000',B'00000000',B'00000000',B'00000000',B'00000000',B'00000000',B'00000000',B'00000000'
	DT	B'11111111',B'11111111',B'11111111',B'11111111',B'11111111',B'11111111',B'11111111',B'11111111'
	DT	B'11111111',B'11111111',B'11111111',B'11111111',B'11111111',B'11111111',B'11111111',B'11111111'
	DT	B'11111111',B'11111111',B'11111111',B'11111111',B'11111111',B'11111111',B'11111111',B'11111111'
	DT	B'11111111',B'11111111',B'11111111',B'11111111',B'11111111',B'11111111',B'11111111',B'11111111'
	DT	B'11111111',B'11111111',B'11111111',B'11111111',B'11111111',B'11111111',B'11111111',B'11111111'
	DT	B'11111111',B'11111111',B'11111111',B'11111111',B'11111111',B'11111111',B'11111111',B'11111111'
	DT	B'11111111',B'11111111',B'11111111',B'11111111',B'11111111',B'11111111',B'11111111',B'11111111'
	DT	B'11111111',B'11111111',B'11111111',B'11111111',B'11111111',B'11111111',B'11111111',B'11111111'
	MOVWF	PCL

; Bit reverse table
	ORG	H'1800'
BITREV	RETLW	B'00000000'
	RETLW	B'10000000'
	RETLW	B'01000000'
	RETLW	B'11000000'
	RETLW	B'00100000'
	RETLW	B'10100000'
	RETLW	B'01100000'
	RETLW	B'11100000'
	RETLW	B'00010000'
	RETLW	B'10010000'
	RETLW	B'01010000'
	RETLW	B'11010000'
	RETLW	B'00110000'
	RETLW	B'10110000'
	RETLW	B'01110000'
	RETLW	B'11110000'
	RETLW	B'00001000'
	RETLW	B'10001000'
	RETLW	B'01001000'
	RETLW	B'11001000'
	RETLW	B'00101000'
	RETLW	B'10101000'
	RETLW	B'01101000'
	RETLW	B'11101000'
	RETLW	B'00011000'
	RETLW	B'10011000'
	RETLW	B'01011000'
	RETLW	B'11011000'
	RETLW	B'00111000'
	RETLW	B'10111000'
	RETLW	B'01111000'
	RETLW	B'11111000'
	RETLW	B'00000100'
	RETLW	B'10000100'
	RETLW	B'01000100'
	RETLW	B'11000100'
	RETLW	B'00100100'
	RETLW	B'10100100'
	RETLW	B'01100100'
	RETLW	B'11100100'
	RETLW	B'00010100'
	RETLW	B'10010100'
	RETLW	B'01010100'
	RETLW	B'11010100'
	RETLW	B'00110100'
	RETLW	B'10110100'
	RETLW	B'01110100'
	RETLW	B'11110100'
	RETLW	B'00001100'
	RETLW	B'10001100'
	RETLW	B'01001100'
	RETLW	B'11001100'
	RETLW	B'00101100'
	RETLW	B'10101100'
	RETLW	B'01101100'
	RETLW	B'11101100'
	RETLW	B'00011100'
	RETLW	B'10011100'
	RETLW	B'01011100'
	RETLW	B'11011100'
	RETLW	B'00111100'
	RETLW	B'10111100'
	RETLW	B'01111100'
	RETLW	B'11111100'
	RETLW	B'00000010'
	RETLW	B'10000010'
	RETLW	B'01000010'
	RETLW	B'11000010'
	RETLW	B'00100010'
	RETLW	B'10100010'
	RETLW	B'01100010'
	RETLW	B'11100010'
	RETLW	B'00010010'
	RETLW	B'10010010'
	RETLW	B'01010010'
	RETLW	B'11010010'
	RETLW	B'00110010'
	RETLW	B'10110010'
	RETLW	B'01110010'
	RETLW	B'11110010'
	RETLW	B'00001010'
	RETLW	B'10001010'
	RETLW	B'01001010'
	RETLW	B'11001010'
	RETLW	B'00101010'
	RETLW	B'10101010'
	RETLW	B'01101010'
	RETLW	B'11101010'
	RETLW	B'00011010'
	RETLW	B'10011010'
	RETLW	B'01011010'
	RETLW	B'11011010'
	RETLW	B'00111010'
	RETLW	B'10111010'
	RETLW	B'01111010'
	RETLW	B'11111010'
	RETLW	B'00000110'
	RETLW	B'10000110'
	RETLW	B'01000110'
	RETLW	B'11000110'
	RETLW	B'00100110'
	RETLW	B'10100110'
	RETLW	B'01100110'
	RETLW	B'11100110'
	RETLW	B'00010110'
	RETLW	B'10010110'
	RETLW	B'01010110'
	RETLW	B'11010110'
	RETLW	B'00110110'
	RETLW	B'10110110'
	RETLW	B'01110110'
	RETLW	B'11110110'
	RETLW	B'00001110'
	RETLW	B'10001110'
	RETLW	B'01001110'
	RETLW	B'11001110'
	RETLW	B'00101110'
	RETLW	B'10101110'
	RETLW	B'01101110'
	RETLW	B'11101110'
	RETLW	B'00011110'
	RETLW	B'10011110'
	RETLW	B'01011110'
	RETLW	B'11011110'
	RETLW	B'00111110'
	RETLW	B'10111110'
	RETLW	B'01111110'
	RETLW	B'11111110'
	RETLW	B'00000001'
	RETLW	B'10000001'
	RETLW	B'01000001'
	RETLW	B'11000001'
	RETLW	B'00100001'
	RETLW	B'10100001'
	RETLW	B'01100001'
	RETLW	B'11100001'
	RETLW	B'00010001'
	RETLW	B'10010001'
	RETLW	B'01010001'
	RETLW	B'11010001'
	RETLW	B'00110001'
	RETLW	B'10110001'
	RETLW	B'01110001'
	RETLW	B'11110001'
	RETLW	B'00001001'
	RETLW	B'10001001'
	RETLW	B'01001001'
	RETLW	B'11001001'
	RETLW	B'00101001'
	RETLW	B'10101001'
	RETLW	B'01101001'
	RETLW	B'11101001'
	RETLW	B'00011001'
	RETLW	B'10011001'
	RETLW	B'01011001'
	RETLW	B'11011001'
	RETLW	B'00111001'
	RETLW	B'10111001'
	RETLW	B'01111001'
	RETLW	B'11111001'
	RETLW	B'00000101'
	RETLW	B'10000101'
	RETLW	B'01000101'
	RETLW	B'11000101'
	RETLW	B'00100101'
	RETLW	B'10100101'
	RETLW	B'01100101'
	RETLW	B'11100101'
	RETLW	B'00010101'
	RETLW	B'10010101'
	RETLW	B'01010101'
	RETLW	B'11010101'
	RETLW	B'00110101'
	RETLW	B'10110101'
	RETLW	B'01110101'
	RETLW	B'11110101'
	RETLW	B'00001101'
	RETLW	B'10001101'
	RETLW	B'01001101'
	RETLW	B'11001101'
	RETLW	B'00101101'
	RETLW	B'10101101'
	RETLW	B'01101101'
	RETLW	B'11101101'
	RETLW	B'00011101'
	RETLW	B'10011101'
	RETLW	B'01011101'
	RETLW	B'11011101'
	RETLW	B'00111101'
	RETLW	B'10111101'
	RETLW	B'01111101'
	RETLW	B'11111101'
	RETLW	B'00000011'
	RETLW	B'10000011'
	RETLW	B'01000011'
	RETLW	B'11000011'
	RETLW	B'00100011'
	RETLW	B'10100011'
	RETLW	B'01100011'
	RETLW	B'11100011'
	RETLW	B'00010011'
	RETLW	B'10010011'
	RETLW	B'01010011'
	RETLW	B'11010011'
	RETLW	B'00110011'
	RETLW	B'10110011'
	RETLW	B'01110011'
	RETLW	B'11110011'
	RETLW	B'00001011'
	RETLW	B'10001011'
	RETLW	B'01001011'
	RETLW	B'11001011'
	RETLW	B'00101011'
	RETLW	B'10101011'
	RETLW	B'01101011'
	RETLW	B'11101011'
	RETLW	B'00011011'
	RETLW	B'10011011'
	RETLW	B'01011011'
	RETLW	B'11011011'
	RETLW	B'00111011'
	RETLW	B'10111011'
	RETLW	B'01111011'
	RETLW	B'11111011'
	RETLW	B'00000111'
	RETLW	B'10000111'
	RETLW	B'01000111'
	RETLW	B'11000111'
	RETLW	B'00100111'
	RETLW	B'10100111'
	RETLW	B'01100111'
	RETLW	B'11100111'
	RETLW	B'00010111'
	RETLW	B'10010111'
	RETLW	B'01010111'
	RETLW	B'11010111'
	RETLW	B'00110111'
	RETLW	B'10110111'
	RETLW	B'01110111'
	RETLW	B'11110111'
	RETLW	B'00001111'
	RETLW	B'10001111'
	RETLW	B'01001111'
	RETLW	B'11001111'
	RETLW	B'00101111'
	RETLW	B'10101111'
	RETLW	B'01101111'
	RETLW	B'11101111'
	RETLW	B'00011111'
	RETLW	B'10011111'
	RETLW	B'01011111'
	RETLW	B'11011111'
	RETLW	B'00111111'
	RETLW	B'10111111'
	RETLW	B'01111111'
	RETLW	B'11111111'

; Graphics table look-up
GTLU	MOVWF	PCL

; Delay routines
	ORG	H'1FFC'
	ADDLW	H'FF'
	BTFSS	STATUS,Z
	GOTO	DEL4N
	RETURN

	END
